#!/usr/bin/perl
use strict;

#
# $Id: Apache.pm,v 1.18 2004/03/15 06:42:05 matt Exp $
#

package MATT::Apache;

use Carp;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION  = '1.18';

@ISA = qw(Exporter);
@EXPORT = qw(
	InstallApache1
	InstallApache2
	InstallApacheSSLCerts
);
@EXPORT_OK = qw( 
	RemoveOldApacheSources 
	ApacheConfPatch
);

use POSIX qw(uname);
use lib "lib";
use MATT::Utility 1;

=head1 NAME

MATT::Apache - Install Apache 1 or 2 on FreeBSD

=head1 SYNOPSIS

Builds a high performance statically compiled web server with SSL, PHP, and Perl support.


=head1 DESCRIPTION

A couple functions used to build Apache with all the dependencies and a few things I like such as: 

mm, gettext, libtool, p5-libwww, cronolog

=cut


sub InstallApache2
{

=head2	InstallApache2

use MATT::Apache;
InstallApache2();

Builds Apache from sources with DSO for all modules. Also installs mod_perl2 and mod_php4.

Currently tested on FreeBSD and Mac OS X. On FreeBSD, the lang/php4 version is installed. It installs both the PHP cli and mod_php4. This is done because the SpamAssassin + SQL module requires pear-DB and the pear-DB port thinks it needs the lang/php4 port installed. There are other ports which also have this requirement so it's best to just have it installed.

This script also builds default SSL certificates, based on your preferences in openssl.cnf (usually in /etc/ssl) and makes a few tweaks to your httpd.conf (for using PHP & perl scripts). 

=cut

	my ($conf) = @_;

	if ( (uname)[0] eq "FreeBSD" ) 
	{
		use MATT::FreeBSD 1;

		print "\n";
		MATT::FreeBSD::InstallPackage ("p5-libwww");
		MATT::FreeBSD::InstallPackage ("apache2", "apache-2");
		if ( $] < 5.006 ) 
		{
			MATT::FreeBSD::InstallPort ("perl5", "lang", "", "perl-5");
			MATT::Utility::SysCmd ("/usr/local/bin/use.perl port");
		};

		MATT::FreeBSD::InstallPort ("cronolog", "sysutils");

		if ( $conf->{'package_install_method'} eq "packages" ) {
			unless ( MATT::FreeBSD::InstallPackage("bison") ) {
				MATT::FreeBSD::InstallPort ("bison", "devel");
			};
			unless ( MATT::FreeBSD::InstallPackage("gd") ) {
				MATT::FreeBSD::InstallPort ("gd", "graphics");
			};
		} else {
			MATT::FreeBSD::InstallPort ("bison", "devel");
			MATT::FreeBSD::InstallPort ("gd", "graphics");
		};

		MATT::FreeBSD::InstallPort ("php4",     "lang", undef, undef, "WITH_APACHE2");
		#MATT::FreeBSD::InstallPort ("mod_php4", "www", undef, undef, "WITH_APACHE2");

		if ( $conf->{'install_apache2_modperl'} ) {
			MATT::FreeBSD::InstallPort ("mod_perl2", "www");
		};

		InstallApacheSSLCerts();
		ApacheConfPatch();
	}
	elsif ( (uname)[0] eq "Darwin" ) 
	{
		unless ( -d "/usr/local/src") { mkdir("/usr/local/src", 0755) };
		unless ( -d "/usr/local/src/www") { mkdir("/usr/local/src/www", 0755) };
		chdir("/usr/local/src/www");
		
		my $apache   = "httpd-2.0.47";
		my $mod_perl = "mod_perl-1.99_10";
		my $mod_php  = "php-4.3.3";
		
		unless ( -e "$apache.tar.gz" ) {
			MATT::Utility::FetchFile("http://www.apache.org/dist/httpd/$apache.tar.gz");
		};
		
		unless ( -e "$mod_perl.tar.gz" ) {
			MATT::Utility::FetchFile("http://perl.apache.org/dist/$mod_perl.tar.gz");
		};

		unless ( -e "$mod_php.tar.gz" ) {
			MATT::Utility::FetchFile("http://us2.php.net/distributions/$mod_php.tar.gz");
		};
		
		foreach my $package ( $apache, $mod_perl, $mod_php) 
		{
			if ( -d $package )
			{
				my $r = MATT::Utility::SourceWarning($package, 1);
				unless ($r) { croak "sorry, I can't continue.\n"; };
			};
			my $tar = MATT::Utility::FindTheBin("tar");
			MATT::Utility::SysCmd("$tar -xzf $package.tar.gz");
		};
	
		if ( -d $apache ) 
		{
			chdir($apache);
			SysCmd("./configure --enable-layout=Darwin --enable-modules=all --enable-mods-shared=all --enable-so");
			SysCmd("make");
			SysCmd("make install");
		};
		print "Darwin eh?\n";
	}
	else
	{
		print "sorry, not yet on " . (uname)[0] . "\n";
	}
};

sub InstallApache1($;$) 
{
	my ($src) = @_;

=head2 InstallApache1

use MATT::Apache;
InstallApache1("/usr/local/src")

Builds Apache from sources with DSO for all but mod_perl which must be compiled statically in order to work at all.

Will build Apache in the directory as shown. After compile, the script will show you a few options for testing and completing the installation.

Also installs mod_php4 and mod_ssl.

=cut

	if ( (uname)[0] eq "Darwin" ) 
	{
		print "if you want Apache1, simply follow the instructions here: \n";
		print "http://www.macdevcenter.com/pub/a/mac/2002/12/18/apache_modssl.html\n";
	};

	use File::Copy;
	use MATT::FreeBSD;

	my $apache   = "apache_1.3.28";
	my $mod_perl = "mod_perl-1.28";
	my $mod_ssl  = "mod_ssl-2.8.15-1.3.28";
	my $layout   = "FreeBSD.layout";

	MATT::FreeBSD::InstallPackage("mm");
	MATT::FreeBSD::InstallPackage("gettext");
	MATT::FreeBSD::InstallPackage("libtool");
	MATT::FreeBSD::InstallPackage("apache");
	MATT::FreeBSD::InstallPackage("p5-libwww");
	MATT::FreeBSD::InstallPort   ("cronolog", "sysutils");

	my $log = "/var/log/apache";
	unless ( -d $log) 
	{
		mkdir($log, 0755) or croak "Couldn't create $log: $!\n";
		my $uid = getpwnam("www");
		my $gid = getgrnam("www");
		chown($uid, $gid, $log);
	};

	MATT::Utility::CdSrcDir("$src/www", $src);

	unless ( -e "$apache.tar.gz" ) {
		MATT::Utility::FetchFile("http://www.apache.org/dist/httpd/$apache.tar.gz");
	};

	unless ( -e "$mod_perl.tar.gz" ) {
		MATT::Utility::FetchFile("http://perl.apache.org/dist/$mod_perl.tar.gz");
	};

	unless ( -e "$mod_ssl.tar.gz" ) {
		MATT::Utility::FetchFile("http://www.modssl.org/source/$mod_ssl.tar.gz");
	};

	unless ( -e $layout ) 
	{
		MATT::Utility::FetchFile("http://www.tnpi.biz/internet/www/apache.layout");
		move("apache.layout", $layout);
	};

	RemoveOldApacheSources($apache);

	foreach my $package ( $apache, $mod_perl, $mod_ssl) 
	{
		if ( -d $package )
		{
			my $r = MATT::Utility::SourceWarning($package, 1);
			if (! $r) { croak "sorry, I can't continue.\n"; };
		};
		my $tar = MATT::Utility::FindTheBin("tar");
		MATT::Utility::SysCmd("$tar -xzf $package.tar.gz");
	};

	chdir($mod_ssl);
	MATT::Utility::SysCmd("./configure --with-apache=../$apache --with-ssl=/usr --enable-shared=ssl --with-mm=/usr/local");
	chdir("../$mod_perl");
	MATT::Utility::SysCmd( "perl Makefile.PL DO_HTTPD=1 USE_APACI=1 APACHE_PREFIX=/usr/local EVERYTHING=1 APACI_ARGS='--server-uid=www, --server-gid=www, --enable-module=so --enable-module=most, --enable-shared=max --disable-shared=perl, --enable-module=perl, --with-layout=../$layout:FreeBSD, --without-confadjust'");
	MATT::Utility::SysCmd("make");

	if (-e "../$apache/src/httpd") 
	{
		print <<EOM

Apache build successful, now you must install as follows:

For new installs:

     cd $src/www/$mod_perl
     make test
     cd ../$apache; make certificate TYPE=custom
     rm /usr/local/etc/apache/httpd.conf
     cd ../$mod_perl; make install
     cd /usr/ports/www/mod_php4; make install clean (optional)
     apachectl stop; apachectl startssl

For re-installs:

     cd $src/www/$mod_perl;\n\tmake test
     make install
     cd /usr/ports/www/mod_php4; make install clean (optional)
     apachectl stop; apachectl startssl
EOM
;
	};
};

sub ApacheConfPatch
{

=head2 ApacheConfPatch

Patch apache's default httpd.conf file. See the patch in contrib of Mail::Toaster to see what changes are being made.

=cut

	my ($prefix) = @_;
	
	unless ($prefix) { $prefix = "/usr/local/etc/apache2"; };

	if ( -d $prefix) { chdir($prefix); } else { die	"$prefix doesn't exist!\n"; };

	unless ( -e "$prefix/httpd.conf-2.0.patch")
	{
		print "ApacheConfPatch FAILURE: patch not found!\n";
		return 1;
	};

	my $httpd = "$prefix/httpd.conf";

	if ( -e "$prefix/httpd.conf.orig" ) {
		print "NOTICE: skipping. It appears the patch is already applied!\n";
		return 0;
	};

	if ( -e $httpd )
	{
		if ( SysCmd("patch $httpd $prefix/httpd.conf-2.0.patch") )
		{
			print "NOTICE: patch apply failed!\n";
			return 1;
		} else  {
			print "NOTICE: patch apply success!\n";
			return 0;
		};
	} 
	else 
	{
		print "FAILURE: I couldn't find your httpd.conf!\n";
		return 1;
	};
	return 0;
};

sub RemoveOldApacheSources
{
	my ($apache) = @_;

	my @list = <apache_1.*>;
	foreach my $dir (@list)
	{
		if ( $dir ne $apache && $dir !~ /\.tar\.gz$/ )
		{
			print "deleting: $dir... ";
			rmtree $_ or croak "couldn't delete $_: $!\n";    
			print "done.";
		};       
	};
};

sub InstallApacheSSLCerts
{

=head2 InstallApacheSSLCerts

Builds and installs certificates in the locations that Apache expects them. This allows me to build a SSL enabled web server with a minimal amount of human interaction.


=cut

	my ($type) = @_;

	my $crtdir = "/usr/local/etc/apache2/ssl.crt";
	unless (-d $crtdir) { SysCmd("mkdir -p $crtdir"); };

	my $keydir = "/usr/local/etc/apache2/ssl.key";
	unless (-d $keydir) { SysCmd("mkdir -p $keydir"); };

	if ($type eq "rsa") 
	{
		unless ( -e "$crtdir/server.crt" ) 
		{
			OpenSSLConfigNote();
			InstallRSACert($crtdir, $keydir);
		} else {
			print "InstallApacheSSLCerts: $crtdir/server.crt is already installed!\n";
		};
	} 
	elsif ( $type eq "dsa" ) 
	{
		unless ( -e "$crtdir/server-dsa.crt" ) {
			#OpenSSLConfigNote();
			#InstallDSACert($crtdir, $keydir);
		} else {
			print "InstallApacheSSLCerts: $crtdir/server-dsa.crt is already installed!\n";
		};
	} 
	else {
		unless ( -e "$crtdir/server.crt" ) {
			OpenSSLConfigNote();
			InstallRSACert($crtdir, $keydir);
		} else {
			print "InstallApacheSSLCerts: $crtdir/server.crt is already installed!\n";
		};
		unless ( -e "$crtdir/server-dsa.crt" ) {
#			OpenSSLConfigNote();
#			InstallDSACert($crtdir, $keydir);
		} else {
			print "InstallApacheSSLCerts: $crtdir/server-dsa.crt is already installed!\n";
		};
	};
};

sub OpenSSLConfigNote
{
	print "ATTENTION! ATTENTION!

If you don't like the default values being offered to you, or you 
get tired of typing them in every time you mess with an SSL cert, 
edit your openssl.cnf file. On most platforms, it lives in 
/etc/ssl/openssl.cnf.\n\n";

	sleep 5;
};

sub InstallDSACert
{

=head2 InstallDSACert

Builds and installs a DSA Certificate.

=cut

	my ($crtdir, $keydir) = @_;

	chdir("/usr/local/etc/apache2");

	my $crt    = "server-dsa.crt";
	my $key    = "server-dsa.key";
	my $csr    = "server-dsa.csr";

	#SysCmd("openssl gendsa 1024 > $keydir/$key");
	#SysCmd("openssl req -new -key $keydir/$key -out $crtdir/$csr");
	#SysCmd("openssl req -x509 -days 999 -key $keydir/$key -in $crtdir/$csr -out $crtdir/$crt");

#	use MATT::Perl;
#	LoadModule("Crypt::OpenSSL::DSA", "p5-Crypt-OpenSSL-DSA", "security");
#	require Crypt::OpenSSL::DSA;
#	my $dsa = Crypt::OpenSSL::DSA->generate_parameters( 1024 );
#	$dsa->generate_key;
#	unless ( -e "$crtdir/$crt" ) { $dsa->write_pub_key(  "$crtdir/$crt" ); };
#	unless ( -e "$keydir/$key" ) { $dsa->write_priv_key( "$keydir/$key" ); };
};

sub InstallRSACert
{

=head2 InstallRSACert

Builds and installs a RSA certificate.

	use MATT::Apache;
	InstallRSACert($crtdir, $keydir);

=cut

	my ($crtdir, $keydir) = @_;

	chdir("/usr/local/etc/apache2");

	my $csr    = "server.csr";
	my $crt    = "server.crt";
	my $key    = "server.key";

	SysCmd("openssl genrsa 1024 > $keydir/$key");
	SysCmd("openssl req -new -key $keydir/$key -out $crtdir/$csr");
	SysCmd("openssl req -x509 -days 999 -key $keydir/$key -in $crtdir/$csr -out $crtdir/$crt");
};

1;
__END__


=head2 Dependencies

Matt::Bundle - http://www.tnpi.biz/computing/perl/MATT-Bundle/

=head1 AUTHOR

Matt Simerson <matt@cadillac.net>

=head1 BUGS

None known. Report any to author.

=head1 TODO

Don't export any of the symbols by default. Move all symbols to EXPORT_OK and explicitely pull in the required ones in programs that need them.

=head1 SEE ALSO

http://www.tnpi.biz/internet/www/

MATT::Apache, MATT::DNS, MATT::FreeBSD, MATT::Mysql,
MATT::Passwd, MATT::Perl, MATT::Qmail, MATT::Quota
MATT::SSL, MATT::Utility


=head1 COPYRIGHT

Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut


