#!/usr/bin/perl
use strict;

#
# $Id: Darwin.pm,v 1.1 2004/02/19 15:20:27 matt Exp $
#

package MATT::Darwin;

use Carp;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION = '1.0';

@ISA     = qw(Exporter);
@EXPORT  = qw(
	InstallDarwinPort
	UpdateDarwinPortsTree
);
@EXPORT_OK = qw( 
	IsPackageInstalled
	CheckPortsAge 
);

use lib "lib";
use MATT::Utility;

=head1 NAME

MATT::Darwin - Mac OS X (Darwin) scripting functions

=head1 SYNOPSIS

MATT::Darwin is a grouping of frequently used functions I've written for perl scripts running on MacOS X (Darwin) systems.

=head1 DESCRIPTION

Usage examples for each subroutine are included.

=cut

sub UpdateDarwinPortsTree
{

=head2 UpdateDarwinPortsTree

Updates the Darwin Ports tree (/usr/ports/*).

	use MATT::Darwin;
	UpdateDarwinPortsTree();

=cut

	my ($cvshost);

	print "\n\nUpdateDarwinPorts: You might want to update your ports tree!\n\n";

	if( MATT::Utility::YesOrNo( "\n\nWould you like me to do it for you?:") )
	{   
		my $cvsbin = MATT::Utility::FindTheBin("cvs");
	};
};


sub InstallDarwinPort($)
{

=head2 InstallDarwinPort

	use MATT::Darwin;
	InstallDarwinPort("openldap2");

That's it. Really. Honest. Nothing more. 

=cut

	my ($name) = @_;

#	CheckPortsAge("30");

	print "InstallDarwinPort: installing $name...";

	MATT::Utility::SysCmd( "port install $name");
#	MATT::Utility::SysCmd( "port clean $name");
};

sub CheckPortsAge($;$)
{
	my ($age, $url) = @_;

	$url = "http://www.tnpi.biz/internet/mail/toaster" if (! $url);

	if ( -M "/usr/ports" > $age )
	{
#		UpdateDarwinPortsTree();
	}
	else
	{
		print "CheckPortsAge: Ports file is current (enough).\n";
	};
};

sub IsPackageInstalled
{
	my ($package, $alt) = @_;
	my $r;  

	my $pkg_info = MATT::Utility::FindTheBin("pkg_info");
	my $grep     = MATT::Utility::FindTheBin("grep");

	if ( $alt )
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $alt-`;
		if ( ! $r ) { $r = `$pkg_info | cut -d" " -f1 | $grep $alt`; };
	} else 
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $package-`;
		if ( ! $r ) { $r = `$pkg_info | cut -d" " -f1 | $grep $package`; };
	};
	chomp $r;

	return $r;
};


1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@tnpi.biz>


=head1 BUGS

	None known. Report any to author.


=head1 TODO

	Needs more documentation.


=head1 SEE ALSO

	http://www.tnpi.biz/
	http://www.tnpi.biz/internet/

MATT::Apache, MATT::DNS, MATT::FreeBSD, MATT::Mysql,
MATT::Passwd, MATT::Perl, MATT::Quota, MATT::Utility


=head1 COPYRIGHT

Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut
