#!/usr/bin/perl
use strict;

#
# $Id: Perl.pm,v 1.15 2004/02/14 22:28:12 matt Exp $
#

package MATT::Perl;

use Carp;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION = '1.15';

@ISA       = qw( Exporter );
@EXPORT    = qw( 
	LoadModule
	CheckPerl
	InstallPerl
);
@EXPORT_OK = qw();

use POSIX qw(uname);
use lib "lib";
use MATT::Utility;


=head1 NAME

MATT::Perl - Common Perl functions

=head1 SYNOPSIS

MATT::Perl is a grouping of frequently used functions I've written

=head1 DESCRIPTION

=cut


sub CheckPerl
{
	if ($] < 5.006001) 
	{
		warn qq{\a\a\a
**************************************************************************
**************************************************************************
  Version $] of perl is NOT SUPPORTED by several mail toaster components. 
  You should strongly consider upgrading your perl before continuing. 
  Perl version 5.6.1 is the lowest version supported.

  Press return to continue...
**************************************************************************
**************************************************************************
		};

		if ( (uname)[0] eq "FreeBSD" && -d "/usr/ports/lang/perl5" ) 
		{
			if ( MATT::Utility::YesOrNo("Would you like me to install Perl 5.6 for you") )
			{
				use MATT::FreeBSD;
				InstallPort("perl5", "lang", "perl-5.6");
				MATT::Utility::SysCmd("/usr/local/bin/use.perl port");
			};
		};
	};
};

sub LoadModule($;$$$)
{
	my ($mod, $name, $dir, $warn) = @_;

=head2 LoadModule

	LoadModule ( $mod, $name, $dir, $warn)
	LoadModule ( "CGI.pm", "p5-CGI.pm", "www")

Loads a required Perl module. If the load fails, we attempt to install the required module (rather than failing gracelessly).

  $mod is the name of the module: (ie. LWP::UserAgent)
  $name is the name of the FreeBSD port
  $dir is is the ports category
  $warn - if set, we warn instead of dying upon failure

=cut

	if (eval "require $mod") 
	{
		$mod->import();
	}
	else
	{
		carp "\ncouldn't import $mod: $@\n\n Would you like me to install $mod: \n";
		if ( MATT::Utility::YesOrNo() ) 
		{
			if ( $dir && (uname)[0] eq "FreeBSD" && -e "/usr/ports/$dir/$name" ) 
			{
				use MATT::FreeBSD;
				MATT::FreeBSD::InstallPort($name, $dir);
			}
			else {
				use CPAN;
				install "$mod";
			};
			$mod->import();
		} 
		else 
		{ 
			if ($warn) { carp "\n$mod is required, you have been warned.\n" }
			else       { croak "\nI'm sorry, $mod is required to continue.\n" };
		};
	};
};

sub InstallPerl
{

=head2 InstallPerl

	use MATT::Perl;
	InstallPerl("perl-5.8.1");

currently only works on FreeBSD and Darwin (Mac OS X)

=cut

	if ( (uname)[0] eq "FreeBSD" ) 
	{
		use MATT::FreeBSD;
		MATT::FreeBSD::InstallPort ("perl5", "lang", "", "perl-5");
		MATT::Utility::SysCmd ("/usr/local/bin/use.perl port");

	}
	elsif ( (uname)[0] eq "Darwin" )
	{
		# as directed at http://developer.apple.com/internet/macosx/
		
		my ($perl) = @_;
		unless ( -d "/usr/local/src") { mkdir("/usr/local/src", 0755) };
		chdir("/usr/local/src");

		$perl = "perl-5.8.1" unless ($perl);

		unless ( -e "$perl.tar.gz" ) {
			MATT::Utility::FetchFile("ftp://ftp.perl.org/pub/CPAN/src/$perl.tar.gz");
		};

		if ( -d $perl )
		{
			my $r = MATT::Utility::SourceWarning($perl, 1);
		};
		my $tar = MATT::Utility::FindTheBin("tar");
		MATT::Utility::SysCmd("$tar -xzf $perl.tar.gz");

		chdir($perl);
		my $replace = MATT::Utility::YesOrNo("\n\n    NOTICE!   \n
Apple installs Perl in /usr/bin. I can install it there as well, overwriting Darwin's
supplied Perl (5.6.0) or it can be installed in /usr/local/bin (The BSD Way). Some
would feel that's safer as it leaves Darwin's distributed version of Perl in tact. I
would suggest that's apt to cause you grief later.\n

Shall I overwrite the default Perl?");

		if ( $replace ) { SysCmd("./Configure -de -Dprefix=/usr"); } 
		else            { SysCmd("./Configure -de");               };

		SysCmd("make");
		SysCmd("make test");

		my $install = MATT::Utility::YesOrNo("\n\n
OK, I just finished running \"make test\" and the results are show above. You should see
a success rate somewhere along the lines of 99.7% okay with only a couple failures. If that's
the case, just feed me a y and I'll install perl for you. Select n to cancel.\n\n");

		if ( $install ) 
		{
			my $sudo = "";
			if ( $< ne "0" ) { $sudo = "/usr/bin/sudo -p 'Password for %u@%h:'"; };

			if ($replace) 
			{
				# while this code isn't critical, it will fix the problem
				# posted here: http://dev.perl.org/perl5/news/2002/07/18/580ann/perldelta.html#mac%20os%20x%20dyld%20undefined%20symbols
				use File::Copy;
				my $file = "/Library/Perl/darwin/CORE/libperl.dylib";
				if ( -e $file ) {
					move($file, "$file.old") or carp "failed to remove $file\n";
				};
				$file = "/System/Library/Perl/darwin/CORE/libperl.dylib";
				if ( -e $file ) {
					move($file, "$file.old") or carp "failed to remove $file\n";
				};
				SysCmd("$sudo find /Library/Perl -name '*.bundle' -exec rm {} \;");
				SysCmd("$sudo find /System/Library/Perl -name '*.bundle' -exec rm {} \;");
			};
			SysCmd("$sudo make install");
		};

	};
};


1;
__END__


=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to author.

=head1 TODO

=head1 SEE ALSO

	http://www.tnpi.biz/computing/
	http://www.tnpi.biz/computing/perl/MATT-Bundle/

MATT::Apache, MATT::DNS, MATT::FreeBSD, MATT::Mysql,
MATT::Passwd, MATT::Perl, MATT::Qmail, MATT::Quota
MATT::SSL, MATT::Utility


=head1 COPYRIGHT

Copyright 2003, The Network People, Inc. All Rights Reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

Neither the name of the The Network People, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=cut


