#!/usr/bin/perl
use strict;

#
# $Id: Quota.pm,v 1.11 2003/12/13 04:54:03 matt Exp $
#

package MATT::Quota;

use Carp;
use Exporter;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

$VERSION = '1.10';

@ISA       = qw(Exporter);
@EXPORT    = qw(QuotaSet );
@EXPORT_OK = qw();

use lib "lib";
use MATT::Utility;
use MATT::FreeBSD;

=head1 NAME

	MATT::Quota - Quota Management functions

=head1 SYNOPSIS

MATT::Quota is a grouping of frequently used functions I've written for dealing with user quotas on FreeBSD systems.

=head1 DESCRIPTION

Dependend on FreeBSD port setquota   - /usr/ports/sysutils/setquota

I wrote this a long time ago and it's only here for legacy purposes. You should not use this and instead use Quota (Tom Zoerner) instead. His version is better, more portable, and I'll be moving that way in the future. This is here only for backwards compatibility with some of my scripts.

=cut

sub QuotaSet
{
	my ($quotafs, $user, $quota, $debug) = @_;

=head2 QuotaSet

	QuotaSet ($quotafs, $user, $quota, $debug)

$quotafs is the file system to add the quota to.

$user is the system user.

$quota is the value value (in Megabytes).

=cut

	my $sudo     = MATT::Utility::FindTheBin("sudo");
	my $setquota = MATT::Utility::FindTheBin("setquota");

	if (!-x $setquota)
	{ 
		MATT::FreeBSD::InstallPort("setquota", "/usr/ports/sysutils/setquota");
		$setquota = MATT::Utility::FindTheBin("setquota");
	};
 
	my $quotabump = $quota + 5;
	print "QuotaSet: setting $quotafs quota for user $user to $quota megabytes..." if $debug;
	my $cmd = "$sudo $setquota -u -f $quotafs -bh${quotabump}M -bs${quota}M $user";
	SysCmd($cmd);
	print "done.\n" if $debug;
};

1;
__END__


=head1 AUTHOR

Matt Simerson <matt@cadillac.net>

=head1 BUGS

None known. Report any to matt@cadillac.net.

=head1 TODO

Needs to be deprecated. Currently used in URON::* and mailadmin?

=head1 SEE ALSO

	http://www.tnpi.biz/computing/
	http://www.tnpi.biz/internet/

MATT::Apache, MATT::DNS, MATT::FreeBSD, MATT::Mysql,
MATT::Passwd, MATT::Perl, MATT::Qmail, MATT::Quota
MATT::SSL, MATT::Utility


=head1 COPYRIGHT

Copyright 2003, The Network People, Inc. All Rights Reserved.

=cut
